# -*- coding: utf-8 -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2016 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU Affero General Public License as published by the Free
#  Software Foundation, either version 3 of the License, or (at your option)
#  any later version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
#  more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
SMS -> SMS data import
"""

from __future__ import unicode_literals, absolute_import

from locsms.db import Session

from rattail.db import newimporting as importing
from rattail.util import OrderedDict

from rattail_locsms import smsimporting


class SMSImportHandler(importing.SQLAlchemyImportHandler):
    """
    Handler for SMS -> SMS data import.
    """
    local_title = "LOC SMS"
    local_dbkey = 'default'
    host_title = "LOC SMS"
    host_dbkey = 'host'

    def get_importers(self):
        importers = OrderedDict()
        importers['AccountLink'] = AccountLinkImporter
        importers['Customer'] = CustomerImporter
        return importers

    def make_host_session(self):
        return Session(bind=self.config.locsms_engines[self.host_dbkey])

    def setup(self):
        super(SMSImportHandler, self).setup()
        self.session = Session(bind=self.config.locsms_engines[self.local_dbkey])

    def teardown(self):
        super(SMSImportHandler, self).teardown()
        self.session.close()


class SMSImporter(importing.SQLAlchemyImporter):
    """
    Base class for Rattail -> Rattail data importers.
    """

    @property
    def host_model_class(self):
        return self.model_class

    @property
    def supported_fields(self):
        fields = self.simple_fields

        # TODO: This probably should be configurable etc.?
        # Ignore F902 since that tends to differ in ways we probably never care
        # about, e.g. between SMS host and store databases.
        if 'F902' in fields:
            fields.remove('F902')

        return fields

    def normalize_source_record(self, host_instance):
        return self.normalize_instance(host_instance)

    def create_instance(self, key, data):
        return self.new_instance(key)

    def update_instance(self, instance, data, instance_data=None):
        return instance

    def delete_instance(self, instance):
        return True


class AccountLinkImporter(SMSImporter, smsimporting.model.AccountLinkImporter):
    pass

class CustomerImporter(SMSImporter, smsimporting.model.CustomerImporter):
    pass
